*-----------------------------------------------------------------------
*-- File........: CRPEWrap.Cc version 2.3
*-- Programmer..: Bowen Moursund (CIS: 72662,436), Copyright 1997
*-- Date........: 02/28/97
*-- Notes.......: CRPEWrapper is a dBASE custom control/class that
*--               is designed to encapsulate the Crystal Reports print
*--               engine, CRPE.DLL. See README.TXT for more details.
*-- Written for.: Visual dBASE v5.5
*-- Rev. History: 05/04/95 - v1.05
*--               05/10/95 - v1.01 WIN.INI parsing bug fix
*--               11/01/95 - David Love, modal preview & line spacing
*--               11/21/95 - v1.5 revisions
*--               04/01/96 - v2.0 revisions
*--               09/01/96 - v2.1 revisions, Jim Sare's VdBDEVMODE class
*--               09/09/96 - v2.1a, PEGetWindowHandle prototype repair
*--               10/28/96 - v2.1b, ClassName->GetClassName change
*--               12/15/96 - v2.2, Preview 'window menu' enhancements
*--                        - ResizeOutputWindow modification
*--               02/28/97 - v2.3, Internationalization. Thanks to
*--                          Francois Ghoche for French strings!
*--                        - Base classes implemented for controls
*--               04/29/97 - v2.3a, bug fixes: on error, SetPrintOptions
*-----------------------------------------------------------------------
#include CRPEFont.H
*CLASS CRPEWrapper(oParent, cName) of text(oParent, cName) custom
CLASS CRPEWrapper

    protect pacClassName, pacVersion
    this.pacClassName = "CRPEWRAPPER"
    this.pacVersion = "2.3"

    private cOnError, lOpenSuccess
    cOnError = setto("on error")
    store .t. to lOpenSuccess
    on error lOpenSuccess = .f.
    do case
    case file(_dbwinhome+"crystal\CRPE.DLL")
        *-- look in the Crystal subdirectory first
        load DLL (_dbwinhome+"crystal\CRPE.DLL")
    case file(home()+"CRPE.DLL")
        *-- look in the VdB subdirectory next
        load DLL (home()+"CRPE.DLL")
    case file(_dbwinhome+"CRPE.DLL")
        *-- if running an .EXE, it might be here
        load DLL (_dbwinhome+"CRPE.DLL")
    otherwise
        *-- look everywhere else, starting with working directory
        load DLL CRPE.DLL
    endcase
    do case
    case file(home()+"CRPEWRAP.DLL")
        *-- look in the VdB subdirectory
        load DLL (home()+"CRPEWRAP.DLL")
    case file(_dbwinhome+"CRPEWRAP.DLL")
        *-- if running an .EXE, it might be here
        load DLL (_dbwinhome+"CRPEWRAP.DLL")
    otherwise
        *-- look everywhere else, starting with working directory
        load DLL CRPEWRAP.DLL
    endcase
    if .not. isblank(cOnError)
        on error &cOnError.
    else
        on error
    endif
    this.OpenSuccess = lOpenSuccess

    #include CRPEWrap.H

    *-- open the print engine
    if this.OpenSuccess .and. this.OpenEngine()

        *-- visual properties for the forms designer surface
        this.Text =  resource(23,"CRPEWRAP.DLL")
        this.ColorNormal = "N/RG*"
        this.Height = 1.2
        this.Width = 28
        this.Visible = .f.
        this.Border = .t.

        *-- initialize protected misc. wrapper properties
        protect Err, XUnits, YUnits, PJHandle

        *-- error messaging object
        set procedure to ErrorMsg.Prg additive
        this.Err = new ErrorMessage(new CRPEWrapErrors())

        class::GetPixelUnits()  && XUnits & YUnits
        this.PWParentWnd = .f.  && will hold PW object reference
        this.PJHandle = 0  && will hold handle to print job

        set procedure to Printer.Prg additive

        *-- initialize printer properties objects
        *-- to default Windows printer
        this.PProps = new PrinterProperties()
        this.PPropsSpare = new PrinterProperties()

        *-- Initialize programmable properties
        class::SetPropertiesToDefault()

    else

        this.OpenSuccess = .f.

    endif
    *-- End of constructor

    PROCEDURE GetClassName
    RETURN this.pacClassName

    PROCEDURE Version
    RETURN this.pacVersion

    PROCEDURE SetPropertiesToDefault
    *-- Default programmable properties
    *-- DO NOT call this method while a preview window is open or a
    *-- print job is in progress!

    *-- path and name of CRW report file
    this.ReportFileName = ""
    this.ShortReportFileName = ""

    *-- use driver specified in RPT: NOT YET IMPLEMENTED!
    this.UseRPTDriver = .f.  && no-op

    *-- print destination
    *-- 0 = window
    *-- 1 = printer
    *-- 2 = file
    this.Destination = 0  && default to window

    *-- path and name of print file if Destination = 2
    this.PrintFileName = ""

    *-- number of copies to print, applicable only if Destination = 1
    this.CopiesToPrinter = _pcopies

    *-- PESetPrintOptions properties
    this.UsePrintOptions = .f. && toggle use of the following properties
    this.StartPage = 1
    this.StopPage = 1
    *-- Collation
    *-- 0 = default, embedded in RPT file
    *-- 1 = Collated
    *-- 2 = Uncolated
    this.Collation = 0

    *-- Line Spacing
    this.SetSpacing = 1

    *******************************************************************
    *-- Preview properties that apply to both VdB form previews
    *-- and Crystal window previews.
    *******************************************************************
    this.PWdBWindow = .t.  && VdB window if .T., Crystal window if .F.
    this.PWMDI = .t.  && MDI preview window if .T.
    this.PWText =  resource(73,"CRPEWRAP.DLL")   && Preview window caption
    this.PWDefaultPos = .t.  && Default PW position/size if .T.
    *-- Default preview window size and position, in turpins
    *-- Defaults fill client MDI area in 640x480
    this.PWTop = .5
    this.PWLeft = 1
    this.PWWidth = 103
    this.PWHeight = 20


    *******************************************************************
    *-- Preview window properties that apply only to a VdB
    *-- preview window, i.e. PWdBWindow = .T.
    *******************************************************************
    *-- Positioning of Crystal preview window (in pixels) relative to
    *-- VdB parent form
    #define PW_LEFT 5
    #define PW_TOP 5
    #define PW_RIGHTMARGIN 9
    #define PW_BOTTOMMARGIN 50

    this.PWModal = .f.  && Modal preview window if .T.
    this.PWMenu = .t.   && Preview window menu if .T.
    this.PWCenter = .t. && center preview window


    *******************************************************************
    *-- Preview window properties that apply only to a Crystal
    *-- preview window, i.e. PWdBWindow = .F.
    *******************************************************************
    this.PWDefaultStyle = .t.
    this.PWBorder = .t.
    this.PWSysMenu = .t.
    this.PWMinimize = .f.
    this.PWMaximize = .f.
    this.PWSizeable = .f.
    this.PWScrollbar = 0
    this.PWWindowState = 0
    this.PWShowControls = .t.
    this.PWindow = .f.


    *-- Not programmable, but included in this method for reset
    *-- to defaults.
    this.PWParentWnd = .f.  && will hold PW object reference
    this.PJHandle = 0
    *-- EoP: SetPropertiesToDefault


    PROCEDURE PrintReport
    local lSuccess

    *-- get job handle
    lSuccess = .t.
    lSuccess = class::OpenPrintJob()

    if lSuccess
        do case

        case this.Destination = 1
        *-- output to printer

            lSuccess = class::SelectPrinter()

            if lSuccess
                lSuccess = class::SetLineSpacing()
            endif

            if lSuccess .and. this.UsePrintOptions
                lSuccess = class::SetPrintOptions()
            endif

            if lSuccess
                lSuccess = class::OutputToPrinter()
            endif

            if lSuccess
                lSuccess =  class::StartPrintJob()
            endif

            if lSuccess
                class::CloseJob(this.PJHandle)
            endif

        case this.Destination = 2
        *-- output to disk file
        *--- haven't worked on this yet

        otherwise
        *-- output to print window

            lSuccess = class::SelectPrinter()

            if lSuccess
                lSuccess = class::SetLineSpacing()
            endif

            if lSuccess .and. this.UsePrintOptions
                lSuccess = class::SetPrintOptions()
            endif

            if lSuccess
                if this.PWdBWindow
                    lSuccess = class::OutPutTodBWindow()
                else
                    lSuccess = class::OutPutToWindow()
                endif
            endif

        endcase
    endif

    RETURN lSuccess
    *-- EoP: PrintReport


    PROCEDURE CloseJob(PJHandle)
    PEClosePrintJob(PJHandle)
    *-- EoP: CloseJob


    PROCEDURE Release
    PECloseEngine()
    release DLL CRPE.DLL
    release DLL CRPEWRAP.DLL
    this.PProps.Release()
    this.PPropsSpare.Release()
    release object this
    *-- EoP: Release


    PROCEDURE ErrMsg(nErrorCode, cMessagePrefix)
    this.Err.ErrorCode = nErrorCode
    this.Err.ShowErrMsg("CRPEWrapper ", cMessagePrefix)
    *-- EoP: ErrMsg


    *-- Use this rather than the dBASE native ChoosePrinter()
    *-- VdBooks method only
    PROCEDURE ChoosePrinter
    local lSuccess
    lSuccess = chooseprinter()
    if lSuccess
        this.Pdriver = left(_pdriver, at(",", _pdriver) - 1)
        this.Pname = right(_pdriver, len(_pdriver) - at(",", _pdriver))
        this.Port = setto("printer")
    endif
    RETURN lSuccess
    *-- EoP: ChoosePrinter


    PROCEDURE OpenEngine
    local lSuccess
    lSuccess = .t.
    if .not. PEOpenEngine()
        this.Err.Message =  resource(95,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), "")
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EoP: OpenEngine


    PROCEDURE OpenPrintJob
    local lSuccess
    lSuccess = .t.
    set talk off
    class::GetShortReportFileName()
    this.PJHandle = PEOpenPrintJob(this.ShortReportFileName)
    if this.PJHandle = 0
        this.Err.Message =  resource(96,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.ReportFileName+": ")
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EoP: OpenPrintJob


    PROCEDURE GetShortReportFileName
    cOldFullPath = set("FULLPATH")
    set fullpath on
    if file(this.ReportFileName)
        this.ShortReportFileName = fshortname(this.ReportFileName)
    else
        this.ShortReportFileName = ""
    endif
    set fullpath &cOldFullPath.
    *-- EoP: GetShortReportFileName


    PROCEDURE StartPrintJob
    local lSuccess
    lSuccess = .t.
    if .not. PEStartPrintJob(this.PJHandle, .t.)
        this.Err.Message =  resource(102,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.ReportFileName+": ")
        class::CloseJob(this.PJHandle)
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EoP: StartPrintJob


    PROCEDURE SelectPrinter
    *-- See the PrinterProperties class in Printer.Prg for
    *-- the details concerning the PProps properties passed as
    *-- parameters to PESelectPrinter
    local lSuccess
    lSuccess = .t.
    this.PProps.SetVdBDevModeFields()
    if .not. PESelectPrinter(this.PJHandle,;
                             this.PProps.PrinterDriver,;
                             this.PProps.PrinterName,;
                             this.PProps.PrinterPort,;
                             this.PProps.DevModeParm)
        this.Err.Message =  resource(99,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.PProps.PrinterName+": ")
        class::CloseJob(this.PJHandle)
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EoP: Selectprinter


    PROCEDURE OutputToPrinter
    local lSuccess
    lSuccess = .t.
    if .not. PEOutputToPrinter(this.PJHandle, this.CopiesToPrinter)
        this.Err.Message =  resource(97,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.ReportFileName+": ")
        class::CloseJob(this.PJHandle)
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EoP: OutputToPrinter


    PROCEDURE SetPrintOptions
    local lSuccess
    private cStructure
    lSuccess = .t.

    #define STORE8(nVal) (Chr(BitAnd(nVal, 255)))
    #define STORE16(nVal) STORE8(nval) + STORE8(BitRShift(nVal, 8))
    cStructure = STORE16(10)+;
                 STORE16(this.StartPage)+;
                 STORE16(this.StopPage)+;
                 STORE16(this.CopiesToPrinter)+;
                 STORE16(this.Collation)

    if .not. PESetPrintOptions(this.PJHandle, cStructure)
        this.Err.Message =  resource(101,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.ReportFileName+": ")
        class::CloseJob(this.PJHandle)
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EoP: SetPrintOptions


    PROCEDURE OutputToWindow  && Output to Crystal PWindow
    local lSuccess, nStyle, nParentHandle, nLeft, nTop, nWidth, nHeight
    lSuccess = .t.
    *-- Next line doesn't always produce the expected results
    nStyle = class::CalcStyle()
    if this.PWDefaultPos
        store CW_USEDEFAULT to nLeft, nTop, nWidth, nHeight
    else
        *-- Convert units to pixels
        nLeft = this.PWLeft * this.XUnits
        nTop = this.PWTop * this.YUnits
        nWidth = this.PWWidth * this.XUnits
        nHeight = this.PWHeight * this.YUnits
    endif
    nParentHandle = iif(this.PWMDI, _app.framewin.hWnd, 0)
    if .not. PEOutputToWindow(this.PJHandle,;
                              this.PWText,;
                              nLeft,;
                              nTop,;
                              nWidth,;
                              nHeight,;
                              nStyle,;
                              nParentHandle)
        this.Err.Message =  resource(98,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.ReportFileName+": ")
        class::CloseJob(this.PJHandle)
        lSuccess = .f.
    endif

    if lSuccess
        PEShowPrintControls(this.PJHandle, this.PWShowControls)
    endif

    if lSuccess
        lSuccess =  class::StartPrintJob()
    endif

    RETURN lSuccess
    *-- EoP: OutputToWindow


    PROCEDURE OutputTodBWindow  && Output to dBASE form window
    local lSuccess, oParentWnd
    lSuccess = .t.

    local oParentWnd
    oParentWnd = new PWindowForm()
    oParentWnd.RptDriver = this
    oParentWnd.MDI = this.PWMDI
    if this.PWMenu
        new PWindowMenu(oParentWnd,"Root")
    endif

    *-- Special segment for VdBooks application
    *-- Ignore this <g>
    if type("_app.App.Module") == "C"
        if _app.App.Module = "LEDGER"
            do LedgRpts.Mnu with oParentWnd
        endif
    endif

    this.PWParentWnd = oParentWnd

    oParentWnd.PJHandle = this.PJHandle

    if .not. this.PWDefaultPos
        oParentWnd.top = this.PWTop
        oParentWnd.left = this.PWLeft
        oParentWnd.width = this.PWWidth
        oParentWnd.height = this.PWHeight
    endif

    oParentWnd.PixelWidth = (this.XUnits * oParentWnd.width) - PW_RIGHTMARGIN
    oParentWnd.PixelHeight = (this.YUnits * oParentWnd.height) - PW_BOTTOMMARGIN

    if this.PWModal
        oParentWnd.MDI = .f.
        oParentWnd.readmodal()
    else
        oParentWnd.open()
    endif

    RETURN lSuccess
    *-- EoP: OutputTodBWindow


    PROCEDURE ResizeOutputWindow(hOutputWindow, nTurpWidth, nTurpHeight)
    #define SWP_NOMOVE      2
    #define SWP_NOZORDER    4
    if type("SetWindowPos") # "FP"
        extern CINT SetWindowPos(CHANDLE,CINT,CINT,CINT,CINT,;
                                 CINT,CINT) User.exe
    endif
    local nWidth, nHeight
    nWidth = (this.XUnits * nTurpWidth) - PW_RIGHTMARGIN
    nHeight = (this.YUnits * nTurpHeight) - PW_BOTTOMMARGIN
    temp = SetWindowPos(hOutputWindow, 0, 0, 0, nWidth, nHeight, ;
                 SWP_NOMOVE+SWP_NOZORDER)
    *-- EoP: ResizeOutputWindow


    PROCEDURE SetLineSpacing
    *-- This procedure sets the line height of the detail band (dwl)
    local lSuccess, nLineHeight, nLineAscent
    *-- PE_TWIPSPERLINE is defined in crpewrap.h as 240 (dwl)
    store PE_TWIPSPERLINE * this.SetSpacing to ;
       nLineHeight, nLineAscent
    lSuccess = .t.
    *-- 4000 identifies the detail region
    *-- 0 is the line number
    if .not. PESetLineHeight(this.PJHandle,;
                              4000,;
                              0,;
                              nLineHeight,;
                              nLineAscent)
        this.Err.Message =  resource(100,"CRPEWRAP.DLL")
        class::ErrMsg(PEGetErrorCode(this.PJHandle), ;
                                     this.ReportFileName+": ")
        class::CloseJob(this.PJHandle)
        lSuccess = .f.
    endif
    RETURN lSuccess
    *-- EOP:  SetLineSpacing


    PROCEDURE CalcStyle
    local nStyle
    nStyle = 0  && default style
    if .not. this.PWDefaultStyle
        nStyle = iif(this.PWBorder,WS_BORDER,nStyle)
        nStyle = iif(this.PWSysmenu,bitor(WS_SYSMENU,nStyle), nStyle)
        nStyle = iif(this.PWSizeable,bitor(WS_THICKFRAME,nStyle), nStyle)
        nStyle = iif(this.PWMaximize,bitor(WS_MAXIMIZEBOX,nStyle), nStyle)
        nStyle = iif(this.PWMinimize,bitor(WS_MINIMIZEBOX,nStyle), nStyle)
        if this.PWWindowState = 1
            nStyle = bitor(WS_MINIMIZE, nStyle)
        elseif this.PWWindowState = 2
            nStyle = bitor(WS_MAXIMIZE, nStyle)
        endif
    endif
    RETURN nStyle
    *-- EoP: CalcStyle


    PROCEDURE CancelPrintJob(PJHandle)
    if PJHandle > 0
        PECancelPrintJob(PJHandle)
    endif
    *-- EoP: CancelPrintJob


    *-- This last group of methods is for an open print window
    PROCEDURE ShowFirstPage(PJHandle)
    if PJHandle > 0
        PEShowFirstPage(PJHandle)
    endif
    *-- EoP: ShowFirstPage


    PROCEDURE ShowPreviousPage(PJHandle)
    if PJHandle > 0
        PEShowPreviousPage(PJHandle)
    endif
    *-- EoP: ShowPreviousPage


    PROCEDURE ShowNextPage(PJHandle)
    if PJHandle > 0
        PEShowNextPage(PJHandle)
    endif
    *-- EoP: ShowNextPage


    PROCEDURE ShowLastPage(PJHandle)
    if PJHandle > 0
        PEShowLastPage(PJHandle)
    endif
    *-- EoP: ShowLastPage


    PROCEDURE PrintWindow(PJHandle)
    if PJHandle > 0
        PEPrintWindow(PJHandle, .t.)
    endif
    *-- EoP: ShowFirstPage


    PROCEDURE ZoomPWindow(PJHandle)
    if PJHandle > 0
        PENextPrintWindowMagnification(PJHandle)
    endif
    *-- EoP: ShowFirstPage


    PROCEDURE GetPixelUnits
    *-- Calculate and store Pixels/Turpins
    set procedure to Center.Prg additive
    local fTemp
    fTemp = new form()
    fTemp.MDI = .f.
    new text(fTemp)
    fTemp.top = -300
    fTemp.open()
    this.XUnits = UnitPixelsX(fTemp)
    this.YUnits = UnitPixelsY(fTemp)
    fTemp.close()
    fTemp.release()

ENDCLASS
*-- EoC: CRPEWrapper

CLASS CRPEWrapErrors
    this[500] =  resource(39,"CRPEWRAP.DLL")
    this[501] =  resource(44,"CRPEWRAP.DLL")
    this[502] =  resource(36,"CRPEWRAP.DLL")
    this[504] =  resource(90,"CRPEWRAP.DLL")
    this[507] =  resource(32,"CRPEWRAP.DLL")
    this[512] =  resource(64,"CRPEWRAP.DLL")
    this[513] =  resource(70,"CRPEWRAP.DLL")
    this[515] =  resource(31,"CRPEWRAP.DLL")
    this[517] =  resource(63,"CRPEWRAP.DLL")
    this[520] =  resource(65,"CRPEWRAP.DLL")
    this[521] =  resource(43,"CRPEWRAP.DLL")
    this[522] =  resource(40,"CRPEWRAP.DLL")
    this[523] =  resource(42,"CRPEWRAP.DLL")
    this[524] =  resource(66,"CRPEWRAP.DLL")
    this[525] =  resource(75,"CRPEWRAP.DLL")
    this[526] =  resource(50,"CRPEWRAP.DLL")
    this[532] =  resource(25,"CRPEWRAP.DLL")
    this[533] =  resource(24,"CRPEWRAP.DLL")
    this[534] =  resource(12,"CRPEWRAP.DLL")
    this[537] =  resource(89,"CRPEWRAP.DLL")
    this[538] =  resource(41,"CRPEWRAP.DLL")
ENDCLASS


*-----------------------------------------------------------------------
*-- Program.....: pwindow.wfm
*-- Programmer..: Bowen Moursund (CIS: 72662,436), Copyright 1997
*-- Date........: 02/28/97
*-- Notes.......: This form, a part of the CRPEWrapper package,
*--               contains the Crystal report preview window and
*--               several preview window controls.
*-- Written for.: Visual dBASE v5.5
*-- Rev. History: 05/04/95 - v1.00
*--               11/01/95 - speedtips, modal by D. Love
*--               11/16/95 - redesign to allow MDI, SDI, and modal
*--               02/28/97 - v2.3 revisions
*-----------------------------------------------------------------------
** END HEADER -- do not remove this line*
* Generated on 05/04/1995
*
*LOCAL f
*f = NEW PWINDOWFORM()
*f.Open()

CLASS PWINDOWFORM OF FORM
   set procedure to CRPECC.CC additive
   this.Top = .5
   this.Left = 1
   this.Text =  resource(73,"CRPEWRAP.DLL")
   *this.MenuFile = "PWINDOW.MNU"
   this.Height = 20
   this.Width = 103
   this.OnSize = CLASS::FORM_ONRESIZE
   this.Icon = "RESOURCE REPORT_ICON CRPEWRAP.DLL"
   store .t. to this.Sysmenu,;
                this.Moveable,;
                this.Sizeable,;
                this.EscExit
   store .f. to this.Minimize,;
                this.Maximize


   DEFINE CRPEBUTTON FIRSTBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         Left          2,;
         Width 6,;
         OnClick CLASS::FIRSTONCLICK,;
         Text "",;
         StatusMessage  resource(106,"CRPEWRAP.DLL"),;
         SpeedTip  resource(106,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE FIRST CRPEWRAP.DLL"

   DEFINE CRPEBUTTON PREVIOUSBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         Left          8.166,;
         Width 6,;
         OnClick CLASS::PREVIOUSONCLICK,;
         Text "",;
         StatusMessage  resource(109,"CRPEWRAP.DLL"),;
         SpeedTip  resource(109,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE PREV CRPEWRAP.DLL"

   DEFINE CRPEBUTTON NEXTBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         Left         14.333,;
         Width 6,;
         OnClick CLASS::NEXTONCLICK,;
         Text "",;
         StatusMessage  resource(108,"CRPEWRAP.DLL"),;
         SpeedTip  resource(108,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE NEXT CRPEWRAP.DLL",;
         Default .T.

   DEFINE CRPEBUTTON LASTBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         Left         20.5,;
         Width          6,;
         Text "",;
         OnClick CLASS::LASTONCLICK,;
         StatusMessage  resource(107,"CRPEWRAP.DLL"),;
         SpeedTip  resource(107,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE LAST CRPEWRAP.DLL"

   DEFINE CRPEBUTTON ZOOMBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         DownBitmap "RESOURCE VIEW_DOWN CRPEWRAP.DLL",;
         Left         47.833,;
         OnClick CLASS::ZOOMONCLICK,;
         Text "&Zoom",;
         StatusMessage  resource(113,"CRPEWRAP.DLL"),;
         Speedtip  resource(113,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE VIEW CRPEWRAP.DLL"

   DEFINE CRPEBUTTON PRINTBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         DownBitmap "RESOURCE PRINT_DOWN CRPEWRAP.DLL",;
         Left         66.166,;
         OnClick CLASS::PRINTONCLICK,;
         Text "&Print",;
         StatusMessage  resource(67,"CRPEWRAP.DLL"),;
         Speedtip  resource(67,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE PRINT CRPEWRAP.DLL"

   DEFINE CRPEBUTTON CLOSEBUTTON OF THIS;
       PROPERTY;
         Top         17.5879,;
         Left         84.166,;
         OnClick {;form.close()},;
         Text resource(167,"CRPEWRAP.DLL"),;
         StatusMessage  resource(16,"CRPEWRAP.DLL"),;
         Speedtip  resource(16,"CRPEWRAP.DLL"),;
         UpBitmap "RESOURCE #1005"


    PROCEDURE Open
    class::PreOpen()
    super::open()


    PROCEDURE Readmodal
    class::PreOpen()
    super::readmodal()


    PROCEDURE PreOpen
    set procedure to Center.Prg additive
    if this.RptDriver.PWCenter
        this.top = -300
    endif
    class::SetButtonTop(this.height)
    this.text = this.RptDriver.PWText
    if this.RptDriver.PWMenu
        this.root.File.Close.OnClick = {;form.close()}
        this.root.File.Print.OnClick = class::PrintOnClick
        this.root.Report.Previous_Page.OnClick = class::PreviousOnClick
        this.root.Report.Next_Page.OnClick = class::NextOnClick
        this.root.Report.Top_Page.OnClick = class::FirstOnClick
        this.root.Report.Bottom_Page.OnClick = class::LastOnClick
        this.root.Report.Zoom.OnClick = class::ZoomOnClick
    endif


    PROCEDURE OnOpen
    if this.RptDriver.PWCenter
        CenterForm(this)
    endif
    form.RptDriver.PWParenthWnd = form.hWnd
    if .not. PEOutputToWindow(form.PJHandle,;
                             "",;
                             PW_LEFT,;
                             PW_TOP,;
                             form.PixelWidth,;
                             form.PixelHeight,;
                             WS_BORDER,;
                             form.hWnd)
        form.RtpDriver.Err.Message = ;
                              resource(98,"CRPEWRAP.DLL")
        form.RptDriver.ErrMsg(PEGetErrorCode(form.PJHandle), ;
                                     form.RptDriver.ReportFileName+": ")
        form.close()
    else
        PEShowPrintControls(form.PJHandle,.f.)
        form.RptDriver.StartPrintJob(form.RptDriver)
        this.hOutputWindow = PEGetWindowHandle(form.PJHandle)
    endif


    PROCEDURE OnClose
    form.RptDriver.CloseJob(form.PJHandle)
    form.RptDriver.PWParentWnd = .f.
    form.release()


    PROCEDURE Form_OnResize(nType, nWidth, nHeight)
    class::SetButtonTop(nHeight)
    form.RptDriver.ResizeOutputWindow(this.hOutputWindow, nWidth, nHeight)


    PROCEDURE SetButtonTop(nHeight)
    local oButton
    oButton = form.first
    do
        oButton.top = nHeight - PW_BUTTONBOTTMARGIN
        oButton = oButton.before
    until oButton.name == form.first.name


    PROCEDURE FirstOnClick
    form.RptDriver.ShowFirstPage(form.PJHandle)


    PROCEDURE LastOnClick
    form.RptDriver.ShowLastPage(form.PJHandle)


    PROCEDURE NextOnClick
    form.RptDriver.ShowNextPage(form.PJHandle)


    PROCEDURE PreviousOnClick
    form.RptDriver.ShowPreviousPage(form.PJHandle)


    PROCEDURE PrintOnClick
    form.RptDriver.PrintWindow(form.PJHandle)


    PROCEDURE ZoomOnClick
    form.RptDriver.ZoomPWindow(form.PJHandle)

ENDCLASS
*-- EoF: pwindow.wfm


CLASS PWINDOWMENU(FormObj,Name) OF MENUBAR(FormObj,Name)
*-----------------------------------------------------------------------
*-- Program.....: pwindow.mnu
*-- Programmer..: Bowen Moursund (CIS: 72662,436), Copyright 1995
*-- Date........: 12/12/96
*-- Notes.......: Menu for the CRPEWrapper print preview window
*-- Written for.: Visual dBASE v5.5
*-- Rev. History: 05/04/95 - v1.00
*--               12/15/96 - Window menu enhancements
*-----------------------------------------------------------------------
   #include WindMenu.H

   DEFINE MENU FILE OF THIS;
       PROPERTY;
         Text resource(166,"CRPEWRAP.DLL")

         DEFINE MENU PRINT OF THIS.FILE;
             PROPERTY;
               Text resource(124,"CRPEWRAP.DLL"),;
               Shortcut "CTRL-P"

         DEFINE MENU MENU4 OF THIS.FILE;
             PROPERTY;
               Text "",;
               Separator .T.

         DEFINE MENU CLOSE OF THIS.FILE;
             PROPERTY;
               Text resource(167,"CRPEWRAP.DLL"),;
               Shortcut "CTRL-F4"

   DEFINE MENU REPORT OF THIS;
       PROPERTY;
         Text resource(168,"CRPEWRAP.DLL")

         DEFINE MENU PREVIOUS_PAGE OF THIS.REPORT;
             PROPERTY;
               Text resource(169,"CRPEWRAP.DLL"),;
               Shortcut "PGUP"

         DEFINE MENU NEXT_PAGE OF THIS.REPORT;
             PROPERTY;
               Text resource(170,"CRPEWRAP.DLL"),;
               Shortcut "PGDN"

         DEFINE MENU TOP_PAGE OF THIS.REPORT;
             PROPERTY;
               Text resource(171,"CRPEWRAP.DLL"),;
               Shortcut "CTRL-PGUP"

         DEFINE MENU BOTTOM_PAGE OF THIS.REPORT;
             PROPERTY;
               Text resource(172,"CRPEWRAP.DLL"),;
               Shortcut "CTRL-PGDN"

         DEFINE MENU ZOOM OF THIS.REPORT;
             PROPERTY;
               Text "&Zoom",;
               Shortcut "CTRL-Z"

    if FormObj.MDI .and. .not. FormObj.RptDriver.PWModal
        DEFINE MENU WINDOW OF THIS;
            PROPERTY;
              StatusMessage  resource(72,"CRPEWRAP.DLL"),;
              Text resource(129,"CRPEWRAP.DLL")

              DEFINE MENU CASCADE OF THIS.WINDOW;
                  PROPERTY;
                    StatusMessage  resource(45,"CRPEWRAP.DLL"),;
                    Text "&Cascade",;
                    OnClick CLASS::CASCADE

              DEFINE MENU TILE_HORIZONTALLY OF THIS.WINDOW;
                  PROPERTY;
                    StatusMessage  resource(7,"CRPEWRAP.DLL"),;
                    Text resource(175,"CRPEWRAP.DLL"),;
                    OnClick CLASS::TILEHORZ

              DEFINE MENU TILE_VERTICALLY OF THIS.WINDOW;
                  PROPERTY;
                    StatusMessage  resource(6,"CRPEWRAP.DLL"),;
               Text resource(188,"CRPEWRAP.DLL"),;
               OnClick CLASS::TILEVERT

              DEFINE MENU ARRANGE_ICONS OF THIS.WINDOW;
                  PROPERTY;
                    StatusMessage  resource(5,"CRPEWRAP.DLL"),;
                    Text resource(177,"CRPEWRAP.DLL"),;
                    OnClick CLASS::ARRANGEICONS

             class::GetVdBMDIClientHandle()
             this.windowmenu = this.window
    endif

   DEFINE MENU HELP OF THIS;
       PROPERTY;
         Text resource(178,"CRPEWRAP.DLL"),;
         OnClick {;msgbox("Would you like to volunteer to write it?"+;
                   chr(13)+ resource(14,"CRPEWRAP.DLL") +" :-)",;
                   resource(48,"CRPEWRAP.DLL") )}

   PROCEDURE GetVdBMDIClientHandle
   *-- based on code by Jim Sare
   local cBuffer
   *-- get first dBW child
   _app.hMDIClient = GetWindow(_app.FrameWin.hWND, GW_CHILD)
   *-- search child windows until MDICLIENT window class found
   do while _app.hMDIClient > 0
       cBuffer = space(81)
       GetClassName(_app.hMDIClient, cBuffer, 80)
       if upper(ltrim(rtrim(cBuffer))) == "MDICLIENT"
           exit  && dBW MDIClient window has been found
       endif
       _app.hMDIClient = GetWindow(_app.hMDIClient, GW_HWNDNEXT)
   enddo
   *-- EoP: GetVdBMDIClientHandle

   PROCEDURE Cascade
   SendMessage(_app.hMDIClient, WM_MDICASCADE, MDITILE_SKIPDISABLED, 0)
   *-- EoP: Cascade

   PROCEDURE TileHorz
   SendMessage(_app.hMDIClient, WM_MDITILE, ;
               MDITILE_HORIZONTAL + MDITILE_SKIPDISABLED, 0)
   *-- EoP: TileHorz

   PROCEDURE TileVert
   SendMessage(_app.hMDIClient, WM_MDITILE, ;
               MDITILE_VERTICAL + MDITILE_SKIPDISABLED, 0)
   *-- EoP: TileVert

   PROCEDURE ArrangeIcons
   SendMessage(_app.hMDIClient, WM_MDIICONARRANGE, 0, 0)
   *-- EoP: ArrangeIcons


ENDCLASS
*-- EoC: pwindow.mnu

*-- EoF: crpewrap.cc
